/********************************************************
 * Stack						*
 *	A file implementing a simple stack class	*
 ********************************************************/
#include <stdlib.h>
#include <iostream.h>

const int STACK_SIZE = 100;	// Maximum size of a stack

/********************************************************
 * Stack class						*
 *							*
 * Member functions					*
 *	stack -- initalize the stack.			*
 *	push -- put an item on the stack.		*
 *	pop -- remove an item from the stack.		*
 ********************************************************/
// The stack itself
class stack {
    private:
	int count;		// Number of items in the stack
	int data[STACK_SIZE];	// The items themselves
    public: // ###
	static int stack_count;	// Number of items in the stack
    public:
	// Initialize the stack
	stack(void);

	// Push an item on the stack
	void push(const int item);

	// Pop an item from the stack
	int pop(void);

	// Get the number of items in the stack
	static get_count(void);
};

int stack::stack_count = 0;	// No stacks defined yet

/********************************************************
 * stack::stack -- initialize the stack.		*	
 ********************************************************/
inline stack::stack(void)
{
    count = 0;	// Zero the stack
    ++stack_count;
}
/********************************************************
 * stack::push -- push an item on the stack.		*
 *							*
 * Warning: We do not check for overflow.		*
 *							*
 * Parameters						*
 *	item -- item to put in the stack		*
 ********************************************************/
inline void stack::push(const int item)
{
    data[count] = item;
    ++count;
}
/********************************************************
 * stack::pop -- get an item off the stack.		*
 *							*
 * Warning: We do not check for stack underflow.	*
 *							*
 * Parameters						*
 *	the_stack -- stack to initalize			*
 *							*
 * Returns						*
 * 	The top item from the stack.			*
 ********************************************************/
inline int stack::pop(void)
{
    // Stack goes down by one
    --count;

    // Then we return the top value
    return (data[count]);
}

/********************************************************
 * stack::get_count -- get the number of stacks		*
 ********************************************************/
inline int stack::get_count(void) 
{
    return (stack_count);
}

// A short routine to test the stack
main()
{
    class stack a_stack;	// Stack we want to use
    class stack b_stack;	// Another stack

    // Push three value on the stack
    a_stack.push(1);
    a_stack.push(2);
    a_stack.push(3);

    cout << "Number of active stacks: " << a_stack.get_count() << '\n';
    cout << "Number of active stacks: " << stack::get_count() << '\n';
    // Pop the item from the stack
    cout << "Expect a 3 ->" << a_stack.pop() << '\n';
    cout << "Expect a 2 ->" << a_stack.pop() << '\n';
    cout << "Expect a 1 ->" << a_stack.pop() << '\n';

    b_stack.push(1);

    return (0);
}
